@@ -1,4 +1,4 @@
-{-# Language QuasiQuotes #-}
+{-# Language QuasiQuotes, OverloadedStrings #-}
 {-|
 Module      : TomlSpec
 Description : Unit tests
@@ -13,53 +13,57 @@ specification document).
 module TomlSpec (spec) where
 
 import Data.Map qualified as Map
+import Data.Text (Text)
 import Data.Time (Day)
 import QuoteStr (quoteStr)
 import Test.Hspec (describe, it, shouldBe, shouldSatisfy, Spec)
-import Toml (Value(..), parse, decode, Result(Success))
-import Toml.ToValue (table, (.=))
+import Toml
+import Toml.Schema (table, (.=))
+
+parse_ :: Text -> Either String Table
+parse_ str = forgetTableAnns <$> parse str
 
 spec :: Spec
 spec =
  do describe "comment"
      do it "ignores comments" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             # This is a full-line comment
             key = "value"  # This is a comment at the end of a line
             another = "# This is not a comment"|]
           `shouldBe`
-          Right (table [("another",String "# This is not a comment"),("key",String "value")])
+          Right (table [("another", Text "# This is not a comment"), ("key", Text "value")])
 
     describe "key/value pair"
      do it "supports the most basic assignments" $
-          parse "key = \"value\"" `shouldBe` Right (Map.singleton "key" (String "value"))
+          parse_ "key = \"value\"" `shouldBe` Right (table ["key" .= Text "value"])
 
         it "requires a value after equals" $
-          parse "key = # INVALID"
+          parse_ "key = # INVALID"
           `shouldBe`
           Left "1:16: parse error: unexpected end-of-input"
 
         it "requires newlines between assignments" $
-          parse "first = \"Tom\" last = \"Preston-Werner\" # INVALID"
+          parse_ "first = \"Tom\" last = \"Preston-Werner\" # INVALID"
           `shouldBe`
           Left "1:15: parse error: unexpected bare key"
 
     describe "keys"
      do it "allows bare keys" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             key = "value"
             bare_key = "value"
             bare-key = "value"
             1234 = "value"|]
           `shouldBe`
           Right (table [
-            "1234"     .= "value",
-            "bare-key" .= "value",
-            "bare_key" .= "value",
-            "key"      .= "value"])
+            "1234"     .= Text "value",
+            "bare-key" .= Text "value",
+            "bare_key" .= Text "value",
+            "key"      .= Text "value"])
 
         it "allows quoted keys" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             "127.0.0.1" = "value"
             "character encoding" = "value"
             "ʎǝʞ" = "value"
@@ -67,22 +71,22 @@ spec =
             'quoted "value"' = "value"|]
           `shouldBe`
           Right (table [
-            "127.0.0.1"          .= "value",
-            "character encoding" .= "value",
-            "key2"               .= "value",
-            "quoted \"value\""   .= "value",
-            "ʎǝʞ"                .= "value"])
+            "127.0.0.1"          .= Text "value",
+            "character encoding" .= Text "value",
+            "key2"               .= Text "value",
+            "quoted \"value\""   .= Text "value",
+            "ʎǝʞ"                .= Text "value"])
 
         it "allows dotted keys" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             name = "Orange"
             physical.color = "orange"
             physical.shape = "round"
             site."google.com" = true|]
           `shouldBe`
           Right (table [
-            "name"     .= "Orange",
-            "physical" .= table ["color" .= "orange", "shape" .= "round"],
+            "name"     .= Text "Orange",
+            "physical" .= table ["color" .= Text "orange", "shape" .= Text "round"],
             "site"     .= table ["google.com" .= True]])
 
         it "prevents duplicate keys" $
@@ -98,7 +102,7 @@ spec =
           `shouldBe` Left "2:1: key error: spelling is already assigned"
 
         it "allows out of order definitions" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             apple.type = "fruit"
             orange.type = "fruit"
 
@@ -110,20 +114,20 @@ spec =
           `shouldBe`
           Right (table [
             "apple" .= table [
-                "color" .= "red",
-                "skin"  .= "thin",
-                "type"  .= "fruit"],
+                "color" .= Text "red",
+                "skin"  .= Text "thin",
+                "type"  .= Text "fruit"],
             "orange" .= table [
-                "color" .= "orange",
-                "skin"  .= "thick",
-                "type"  .= "fruit"]])
+                "color" .= Text "orange",
+                "skin"  .= Text "thick",
+                "type"  .= Text "fruit"]])
 
         it "allows numeric bare keys" $
-          parse "3.14159 = 'pi'" `shouldBe` Right (table [
-            "3" .= table [("14159", String "pi")]])
+          parse_ "3.14159 = 'pi'" `shouldBe` Right (table [
+            "3" .= table ["14159" .= Text "pi"]])
 
         it "allows keys that look like other values" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             true = true
             false = false
             1900-01-01 = 1900-01-01
@@ -137,20 +141,20 @@ spec =
 
     describe "string"
      do it "parses escapes" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             str = "I'm a string. \"You can quote me\". Name\tJos\u00E9\nLocation\tSF."|]
           `shouldBe`
-          Right (Map.singleton "str" (String "I'm a string. \"You can quote me\". Name\tJos\xe9\nLocation\tSF."))
+          Right (table ["str" .= Text "I'm a string. \"You can quote me\". Name\tJos\xe9\nLocation\tSF."])
 
         it "strips the initial newline from multiline strings" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             str1 = """
             Roses are red
             Violets are blue"""|]
-          `shouldBe` Right (Map.singleton "str1" (String "Roses are red\nViolets are blue"))
+          `shouldBe` Right (table ["str1" .= Text "Roses are red\nViolets are blue"])
 
         it "strips whitespace with a trailing escape" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             # The following strings are byte-for-byte equivalent:
             str1 = "The quick brown fox jumps over the lazy dog."
 
@@ -168,12 +172,12 @@ spec =
                 """|]
           `shouldBe`
           Right (table [
-            "str1" .= "The quick brown fox jumps over the lazy dog.",
-            "str2" .= "The quick brown fox jumps over the lazy dog.",
-            "str3" .= "The quick brown fox jumps over the lazy dog."])
+            "str1" .= Text "The quick brown fox jumps over the lazy dog.",
+            "str2" .= Text "The quick brown fox jumps over the lazy dog.",
+            "str3" .= Text "The quick brown fox jumps over the lazy dog."])
 
         it "allows quotes inside multiline quoted strings" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             str4 = """Here are two quotation marks: "". Simple enough."""
             str5 = """Here are three quotation marks: ""\"."""
             str6 = """Here are fifteen quotation marks: ""\"""\"""\"""\"""\"."""
@@ -182,10 +186,10 @@ spec =
             str7 = """"This," she said, "is just a pointless statement.""""|]
           `shouldBe`
           Right (table [
-            "str4" .= "Here are two quotation marks: \"\". Simple enough.",
-            "str5" .= "Here are three quotation marks: \"\"\".",
-            "str6" .= "Here are fifteen quotation marks: \"\"\"\"\"\"\"\"\"\"\"\"\"\"\".",
-            "str7" .= "\"This,\" she said, \"is just a pointless statement.\""])
+            "str4" .= Text "Here are two quotation marks: \"\". Simple enough.",
+            "str5" .= Text "Here are three quotation marks: \"\"\".",
+            "str6" .= Text "Here are fifteen quotation marks: \"\"\"\"\"\"\"\"\"\"\"\"\"\"\".",
+            "str7" .= Text "\"This,\" she said, \"is just a pointless statement.\""])
 
         it "disallows triple quotes inside a multiline string" $
           parse [quoteStr|
@@ -193,7 +197,7 @@ spec =
           `shouldBe` Left "1:46: parse error: unexpected '.'"
 
         it "ignores escapes in literal strings" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             # What you see is what you get.
             winpath  = 'C:\Users\nodejs\templates'
             winpath2 = '\\ServerX\admin$\system32\'
@@ -201,13 +205,13 @@ spec =
             regex    = '<\i\c*\s*>'|]
           `shouldBe`
           Right (table [
-            "quoted"   .= "Tom \"Dubs\" Preston-Werner",
-            "regex"    .= "<\\i\\c*\\s*>",
-            "winpath"  .= "C:\\Users\\nodejs\\templates",
-            "winpath2" .= "\\\\ServerX\\admin$\\system32\\"])
+            "quoted"   .= Text "Tom \"Dubs\" Preston-Werner",
+            "regex"    .= Text "<\\i\\c*\\s*>",
+            "winpath"  .= Text "C:\\Users\\nodejs\\templates",
+            "winpath2" .= Text "\\\\ServerX\\admin$\\system32\\"])
 
         it "handles multiline literal strings" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             regex2 = '''I [dw]on't need \d{2} apples'''
             lines  = '''
             The first newline is
@@ -217,17 +221,17 @@ spec =
             '''|]
           `shouldBe`
           Right (table [
-            "lines"  .= "The first newline is\ntrimmed in raw strings.\nAll other whitespace\nis preserved.\n",
-            "regex2" .= "I [dw]on't need \\d{2} apples"])
+            "lines"  .= Text "The first newline is\ntrimmed in raw strings.\nAll other whitespace\nis preserved.\n",
+            "regex2" .= Text "I [dw]on't need \\d{2} apples"])
 
         it "parses all the other escapes" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             x = "\\\b\f\r\U0010abcd"
             y = """\\\b\f\r\u7bca\U0010abcd\n\r\t"""|]
           `shouldBe`
           Right (table [
-            "x" .= "\\\b\f\r\x0010abcd",
-            "y" .= "\\\b\f\r\x7bca\x0010abcd\n\r\t"])
+            "x" .= Text "\\\b\f\r\x0010abcd",
+            "y" .= Text "\\\b\f\r\x7bca\x0010abcd\n\r\t"])
 
         it "rejects out of range unicode escapes" $
           parse [quoteStr|
@@ -242,7 +246,7 @@ spec =
 
     describe "integer"
      do it "parses literals correctly" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             int1 = +99
             int2 = 42
             int3 = 0
@@ -287,7 +291,7 @@ spec =
 
     describe "float"
      do it "parses floats" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             # fractional
             flt1 = +1.0
             flt2 = 3.1415
@@ -307,20 +311,20 @@ spec =
             sf3 = -inf # negative infinity|]
           `shouldBe`
           Right (table [
-            "flt1" .= Float 1.0,
-            "flt2" .= Float 3.1415,
-            "flt3" .= Float (-1.0e-2),
-            "flt4" .= Float 4.9999999999999996e22,
-            "flt5" .= Float 1000000.0,
-            "flt6" .= Float (-2.0e-2),
-            "flt7" .= Float 6.626e-34,
-            "flt8" .= Float 224617.445991228,
-            "sf1"  .= Float (1/0),
-            "sf2"  .= Float (1/0),
-            "sf3"  .= Float (-1/0)])
+            "flt1" .= Double 1.0,
+            "flt2" .= Double 3.1415,
+            "flt3" .= Double (-1.0e-2),
+            "flt4" .= Double 4.9999999999999996e22,
+            "flt5" .= Double 1000000.0,
+            "flt6" .= Double (-2.0e-2),
+            "flt7" .= Double 6.626e-34,
+            "flt8" .= Double 224617.445991228,
+            "sf1"  .= Double (1/0),
+            "sf2"  .= Double (1/0),
+            "sf3"  .= Double (-1/0)])
 
         it "parses nan correctly" $
-          let checkNaN (Float x) = isNaN x
+          let checkNaN (Double' _ x) = isNaN x
               checkNaN _         = False
           in
           parse [quoteStr|
@@ -330,19 +334,19 @@ spec =
             sf6 = -nan # valid, actual encoding is implementation-specific|]
           `shouldSatisfy` \case
             Left{} -> False
-            Right x -> all checkNaN x
+            Right (MkTable x) -> all (checkNaN . snd) x
 
         -- code using Numeric.readFloat can use significant
         -- resources. this makes sure this doesn't start happening
         -- in the future
         it "parses huge floats without great delays" $
-          parse "x = 1e1000000000000"
+          parse_ "x = 1e1000000000000"
           `shouldBe`
-          Right (Map.singleton "x" (Float (1/0)))
+          Right (table ["x" .= Double (1/0)])
 
     describe "boolean"
      do it "parses boolean literals" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             bool1 = true
             bool2 = false|]
           `shouldBe`
@@ -352,7 +356,7 @@ spec =
 
     describe "offset date-time"
      do it "parses offset date times" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             odt1 = 1979-05-27T07:32:00Z
             odt2 = 1979-05-27T00:32:00-07:00
             odt3 = 1979-05-27T00:32:00.999999-07:00
@@ -366,7 +370,7 @@ spec =
 
     describe "local date-time"
      do it "parses local date-times" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             ldt1 = 1979-05-27T07:32:00
             ldt2 = 1979-05-27T00:32:00.999999
             ldt3 = 1979-05-28 00:32:00.999999|]
@@ -384,14 +388,14 @@ spec =
 
     describe "local date"
      do it "parses dates" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             ld1 = 1979-05-27|]
           `shouldBe`
-          Right (Map.singleton "ld1" (Day (read "1979-05-27")))
+          Right (table ["ld1" .= Day (read "1979-05-27")])
 
     describe "local time"
      do it "parses times" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             lt1 = 07:32:00
             lt2 = 00:32:00.999999|]
           `shouldBe`
@@ -401,7 +405,7 @@ spec =
 
     describe "array"
      do it "parses array examples" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             integers = [ 1, 2, 3 ]
             colors = [ "red", "yellow", "green" ]
             nested_arrays_of_ints = [ [ 1, 2 ], [3, 4, 5] ]
@@ -416,21 +420,21 @@ spec =
             ]|]
             `shouldBe`
             Right (table [
-                "colors" .= ["red", "yellow", "green"],
+                "colors" .= [Text "red", Text "yellow", Text "green"],
                 "contributors" .= [
-                    String "Foo Bar <foo@example.com>",
+                    "Foo Bar <foo@example.com>",
                     Table (table [
-                        "email" .= "bazqux@example.com",
-                        "name" .= "Baz Qux",
-                        "url" .= "https://example.com/bazqux"])],
+                        "email" .= Text "bazqux@example.com",
+                        "name" .= Text "Baz Qux",
+                        "url" .= Text "https://example.com/bazqux"])],
                 "integers" .= [1, 2, 3 :: Integer],
                 "nested_arrays_of_ints" .= [[1, 2], [3, 4, 5 :: Integer]],
-                "nested_mixed_array" .= [[Integer 1, Integer 2], [String "a", String "b", String "c"]],
-                "numbers" .= [Float 0.1, Float 0.2, Float 0.5, Integer 1, Integer 2, Integer 5],
-                "string_array" .= ["all", "strings", "are the same", "type"]])
+                "nested_mixed_array" .= [[Integer 1, Integer 2], [Text "a", Text "b", Text "c"]],
+                "numbers" .= [Double 0.1, Double 0.2, Double 0.5, Integer 1, Integer 2, Integer 5],
+                "string_array" .= [Text "all", Text "strings", Text "are the same", Text "type"]])
 
         it "handles newlines and comments" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             integers2 = [
             1, 2, 3
             ]
@@ -445,14 +449,14 @@ spec =
                 "integers3" .= [1, 2 :: Int]])
 
         it "disambiguates double brackets from array tables" $
-          parse "x = [[1]]" `shouldBe` Right (Map.singleton "x" (Array [Array [Integer 1]]))
+          parse_ "x = [[1]]" `shouldBe` Right (table ["x" .= List [List [Integer 1]]])
 
     describe "table"
      do it "allows empty tables" $
-          parse "[table]" `shouldBe` Right (table ["table" .= table []])
+          parse_ "[table]" `shouldBe` Right (table ["table" .= table []])
 
         it "parses simple tables" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             [table-1]
             key1 = "some string"
             key2 = 123
@@ -463,21 +467,21 @@ spec =
           `shouldBe`
           Right (table [
             "table-1" .= table [
-                "key1" .= "some string",
+                "key1" .= Text "some string",
                 "key2" .= Integer 123],
             "table-2" .= table [
-                "key1" .= "another string",
+                "key1" .= Text "another string",
                 "key2" .= Integer 456]])
 
         it "allows quoted keys" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             [dog."tater.man"]
             type.name = "pug"|]
           `shouldBe`
-          Right (table ["dog" .= table ["tater.man" .= table ["type" .= table ["name" .= "pug"]]]])
+          Right (table ["dog" .= table ["tater.man" .= table ["type" .= table ["name" .= Text "pug"]]]])
 
         it "allows whitespace around keys" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             [a.b.c]            # this is best practice
             [ d.e.f ]          # same as [d.e.f]
             [ g .  h  . i ]    # same as [g.h.i]
@@ -490,7 +494,7 @@ spec =
             "j" .= table ["ʞ" .= table ["l" .= table []]]])
 
         it "allows supertables to be defined after subtables" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             # [x] you
             # [x.y] don't
             # [x.y.z] need these
@@ -515,7 +519,7 @@ spec =
           `shouldBe` Left "4:8: key error: apple is a closed table"
 
         it "can add subtables" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             [fruit]
             apple.color = "red"
             apple.taste.sweet = true
@@ -525,7 +529,7 @@ spec =
           Right (table [
             "fruit" .= table [
                 "apple" .= table [
-                    "color" .= "red",
+                    "color" .= Text "red",
                     "taste" .= table [
                         "sweet" .= True],
                         "texture" .= table [
@@ -533,14 +537,14 @@ spec =
 
     describe "inline table"
      do it "parses inline tables" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             name = { first = "Tom", last = "Preston-Werner" }
             point = { x = 1, y = 2 }
             animal = { type.name = "pug" }|]
           `shouldBe`
           Right (table [
-            "animal" .= table ["type" .= table ["name" .= "pug"]],
-            "name"   .= table ["first" .= "Tom", "last" .= "Preston-Werner"],
+            "animal" .= table ["type" .= table ["name" .= Text "pug"]],
+            "name"   .= table ["first" .= Text "Tom", "last" .= Text "Preston-Werner"],
             "point"  .= table ["x" .= Integer 1, "y" .= Integer 2]])
 
         it "prevents altering inline tables with dotted keys" $
@@ -557,6 +561,26 @@ spec =
             type = { edible = false }  # INVALID|]
           `shouldBe` Left "3:1: key error: type is already assigned"
 
+        it "checks that inline keys aren't reassigned" $
+          parse [quoteStr|
+            x = {a = 1, a = 2}|]
+          `shouldBe` Left "1:13: key error: a is already assigned"
+
+        it "checks that inline keys don't overlap with implicit inline tables" $
+          parse [quoteStr|
+            x = {a.b = 1, a = 2}|]
+          `shouldBe` Left "1:15: key error: a is already assigned"
+
+        it "checks for overwrites from other inline tables" $
+          parse [quoteStr|
+            tab = { inner = { dog = "best" }, inner.cat = "worst" }|]
+          `shouldBe` Left "1:35: key error: inner is already assigned"
+
+        it "checks for overlaps of other inline tables" $
+          parse [quoteStr|
+            tbl = { fruit = { apple.color = "red" }, fruit.apple.texture = { smooth = true } }|]
+          `shouldBe` Left "1:42: key error: fruit is already assigned"
+
     describe "array of tables"
      do it "supports array of tables syntax" $
           decode [quoteStr|
@@ -572,18 +596,18 @@ spec =
 
             color = "gray"|]
           `shouldBe`
-          Success mempty (Map.singleton "products" [
+          Success mempty (Map.singleton ("products" :: Text) [
             table [
-              "name" .= "Hammer",
+              "name" .= Text "Hammer",
               "sku"  .= Integer 738594937],
-            Map.empty,
+            table [],
             table [
-                "color" .= "gray",
-                "name"  .= "Nail",
+                "color" .= Text "gray",
+                "name"  .= Text "Nail",
                 "sku"   .= Integer 284758393]])
 
         it "handles subtables under array of tables" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             [[fruits]]
             name = "apple"
 
@@ -607,17 +631,17 @@ spec =
           Right (table [
             "fruits" .= [
                 table [
-                    "name" .= "apple",
+                    "name" .= Text "apple",
                     "physical" .= table [
-                        "color" .= "red",
-                        "shape" .= "round"],
+                        "color" .= Text "red",
+                        "shape" .= Text "round"],
                     "varieties" .= [
-                        table ["name" .= "red delicious"],
-                        table ["name" .= "granny smith"]]],
+                        table ["name" .= Text "red delicious"],
+                        table ["name" .= Text "granny smith"]]],
                 table [
-                    "name" .= "banana",
+                    "name" .= Text "banana",
                     "varieties" .= [
-                        table ["name" .= "plantain"]]]]])
+                        table ["name" .= Text "plantain"]]]]])
 
         it "prevents redefining a supertable with an array of tables" $
           parse [quoteStr|
@@ -642,12 +666,12 @@ spec =
     -- these cases are needed to complete coverage checking on Semantics module
     describe "corner cases"
      do it "stays open" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             [x.y.z]
             [x]
             [x.y]|]
           `shouldBe`
-          parse "x.y.z={}"
+          parse_ "x.y.z={}"
 
         it "stays closed" $
           parse [quoteStr|
@@ -656,20 +680,20 @@ spec =
             [x.y]|] `shouldBe` Left "3:4: key error: y is a closed table"
 
         it "super tables of array tables preserve array tables" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             [[x.y]]
             [x]
             [[x.y]]|]
           `shouldBe`
-          parse "x.y=[{},{}]"
+          parse_ "x.y=[{},{}]"
 
         it "super tables of array tables preserve array tables" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             [[x.y]]
             [x]
             [x.y.z]|]
           `shouldBe`
-          parse "x.y=[{z={}}]"
+          parse_ "x.y=[{z={}}]"
 
         it "detects conflicting inline keys" $
           parse [quoteStr|
@@ -677,7 +701,7 @@ spec =
           `shouldBe` Left "1:14: key error: y is already assigned"
 
         it "handles merging dotted inline table keys" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             t = { a.x.y = 1, a.x.z = 2, a.q = 3}|]
           `shouldBe`
           Right (table [
@@ -685,8 +709,8 @@ spec =
                 "a" .= table [
                     "q" .= Integer 3,
                     "x" .= table [
-                        ("y",Integer 1),
-                        ("z",Integer 2)]]]])
+                        "y" .= Integer 1,
+                        "z" .= Integer 2]]]])
 
         it "disallows overwriting assignments with tables" $
           parse [quoteStr|
@@ -695,20 +719,20 @@ spec =
           `shouldBe` Left "2:2: key error: x is already assigned"
 
         it "handles super super tables" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             [x.y.z]
             [x.y]
             [x]|]
           `shouldBe`
-          parse "x.y.z={}"
+          parse_ "x.y.z={}"
 
         it "You can dot into open supertables" $
-          parse [quoteStr|
+          parse_ [quoteStr|
             [x.y.z]
             [x]
             y.q = 1|]
           `shouldBe`
-          parse "x.y={z={},q=1}"
+          parse_ "x.y={z={},q=1}"
 
         it "dotted tables close previously open tables" $
           parse [quoteStr|
